#!/bin/bash
#
#
# Plot the results from unblur using gnuplot
#
# Alexis Rohou, July 2014
#
# Copyright 2014 Howard Hughes Medical Institute. All rights reserved.
# Use is subject to Janelia Farm Research Campus Software Copyright 1.1
# license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
#



# Parse arguments
if [ $# -ne 1 ]
then
	echo "Usage: `basename $0` output_from_unblur_shifts.txt"
	exit 65
fi

input_fn=$1
output_fn=${1%%.???}.pdf
output_fn_global_scaling=${1%%.???}_global_scaling.pdf

# Check whether gnuplot is available
if ! hash gnuplot 2>/dev/null; then
	echo "Gnuplot was not found on your system. Please make sure it is installed."
	exit -1
fi


# Define a neat function to transpose a data file (columns-rows)
function transpose_ignore_comments {
gawk '
{
	if ($1 ~ /^#/) {
		#print $0
	} else {
		for (i=1; i<=NF; i++)  {
       		a[NR,i] = $i
    	}
	}
}
NF>p { p = NF }
END {    
    for(j=1; j<=p; j++) {
        str=a[1,j]
        for(i=2; i<=NR; i++){
            str=str" "a[i,j];
        }
        print str
    }
}' $1
}


# Define a function to take a file with data arranged in columns
# (x1, y1, x2, y2, ...) and compute estimated movement during frame
function compute_shift_during_frame {
gawk '

	BEGIN {
		split($0,current_line," ");	
	}
	NR==2 {
		for (i=1; i<=NF; i++)
			previous_line[$i] = current_line[$i];
		
		split($0,current_line," ");
		for (i=1; i<=NF; i=i+2)
			printf "%6.3f ",  sqrt((current_line[i]-previous_line[i])^2+(current_line[i+1] -previous_line[i+1])^2)

		printf "\n"

	}
	NR>2 {
		for (i=1;i<=NF;i++)
			previous_previous_line[$i] = previous_line[$i];
			previous_line[$i] = current_line[$i];

		split($0,current_line," ");
		for (i=1; i<=NF; i=i+2)
			printf "%6.3f ", 0.5*sqrt((current_line[i] -previous_line[i]         )^2+(current_line[i+1] -previous_line[i+1]         )^2) + \
			                 0.5*sqrt((previous_line[i]-previous_previous_line[i])^2+(previous_line[i+1]-previous_previous_line[i+1])^2) 
		
		printf "\n"	
	}
	END {
		for (i=1; i<=NF; i++)
			previous_line[$i] = current_line[$i];
		
		split($0,current_line," ");
		for (i=1; i<=NF; i=i+2)
			printf "%6.3f ",  sqrt((current_line[i]-previous_line[i])^2+(current_line[i+1] -previous_line[i+1])^2)

		printf "\n"

	}
' $1
}


temp_fn=$(mktemp)
temp_shifts_fn=$(mktemp)

# CTFFind outputs data in lines, but gnuplot wants things in columns
transpose_ignore_comments $input_fn > $temp_fn


# Let's grab useful values
pixel_size=$(gawk 'match($0,/Pixel size \(A\): ([0-9.]*)/,a) {print a[1]}' $input_fn)
number_of_micrographs=$(gawk 'match($0,/Number of micrographs: ([0-9]*)/,a) {print a[1]}' $input_fn)
lines_per_micrograph=2

# Compute shifts during frames
compute_shift_during_frame $temp_fn > $temp_shifts_fn

cp $temp_fn temp.txt
cp $temp_shifts_fn temp_shifts.txt

# Let's compute the shifts using Julia
julia <<eof
frame_positions = readdlm("$temp_fn",' ');
typeof(frame_positions)
eof


# Run Gnuplot
gnuplot > /dev/null 2>&1  <<EOF
set border linewidth 1.5
set terminal pdfcairo size 5.0cm,5.0cm enhanced font 'Arial,9'
set output '$output_fn'

# color definitions
set style line 1 lc rgb '#0060ad' lt 1 lw 2 pt 7 ps 0.1 # --- blue
set style line 2 lc rgb 'red'     lt 1 lw 1 pt 7  # --- red
set style line 3 lc rgb 'orange'  lt 3 lw 1 pt 7 # --- orange
set style line 4 lc rgb 'light-blue' lt 1 lw 2 pt 7 # --- light blue
set style line 5 lc rgb 'green'   lt 1 lw 2 pt 7
set style line 6 lc rgb 'gray'    lt 1 lw 2 pt 7

unset key
set size square
set size ratio-1
set xrange [] writeback
set yrange [] writeback

set xlabel 'X shift (Å)'
set ylabel 'Y shift (Å)'

# define min and max functions
min(A,B) = A < B ? A : B
max(A,B) = A > B ? A : B
minfour(A,B,C,D) = min(A,min(B,min(C,D)))
maxfour(A,B,C,D) = max(A,max(B,max(C,D)))
maxabsfour(A,B,C,D) = max(abs(A),max(abs(B),max(abs(C),abs(D))))

absmax_global = 0.0

do for [current_micrograph=1:$number_of_micrographs] {
	set title 'Micrograph '.current_micrograph.' of $number_of_micrographs'
	num_column_x = 1+(current_micrograph-1)*$lines_per_micrograph
	num_column_y = 2+(current_micrograph-1)*$lines_per_micrograph
	set xrange[*:*]
	set yrange[*:*]
	stats '$temp_fn'    using	(column(num_column_x)*$pixel_size):(column(num_column_y)*$pixel_size)
	absmax = maxabsfour(STATS_min_x,STATS_max_x,STATS_min_y,STATS_max_y)*1.1
	absmax_global = max(absmax_global,absmax)
	set xrange[-absmax:absmax]
	set yrange[-absmax:absmax]
	plot '$temp_fn'       using (column(num_column_x)*$pixel_size):(column(num_column_y)*$pixel_size) w linespoints ls 1, \
		 '' every ::0::0  using (column(num_column_x)*$pixel_size):(column(num_column_y)*$pixel_size) w points ls 1 ps 0.2
}

set output '$output_fn_global_scaling'
set xrange[-absmax_global:absmax_global]
set yrange[-absmax_global:absmax_global]
do for [current_micrograph=1:$number_of_micrographs] {
	set title 'Micrograph '.current_micrograph.' of $number_of_micrographs'
	num_column_x = 1+(current_micrograph-1)*$lines_per_micrograph
	num_column_y = 2+(current_micrograph-1)*$lines_per_micrograph
	plot '$temp_fn'       using (column(num_column_x)*$pixel_size):(column(num_column_y)*$pixel_size) w linespoints ls 1, \
		 '' every ::0::0  using (column(num_column_x)*$pixel_size):(column(num_column_y)*$pixel_size) w points ls 1 ps 0.2
}
EOF

temp_pdf_fn=$(mktemp)

# Fix the title of the PDF file
if hash pdftk 2>/dev/null; then
	echo "InfoKey: Title" > $temp_fn
	echo "InfoValue: $1" >> $temp_fn
	mv ${output_fn} $temp_pdf_fn
	pdftk $temp_pdf_fn update_info $temp_fn output $output_fn
else
	echo "pdftk was not found on your system. Please install it to get improved metadata in your output PDF file."
fi

rm -f $temp_fn $temp_pdf_fn

# Make a PNG version
#convert -density 300 -flatten ${output_fn} ${output_fn%%.pdf}.png

# Let the user know where to find results
echo " "
echo "Generated plots of 1D profiles and fits: ${output_fn}"
echo " "
